# tres/Scripts/tharshift1.cl
# July 21, 2008
# By Doug Mink

# THARSHIFT1 - Find the correct pixel shift from ThAr spectra
#	       for one object spectrum

procedure tharshift1 (spectrum)

string	spectrum=""	{prompt="Spectrum for which to find a pixel shift"}
real	shift=0.0	{prompt="Pixel shift (returned)"}
file	refpath=""	{prompt="Reference ID ThAr file path"}
string	refmode="interp" {prompt="Method (closest, interp, fit, reference)"}
bool    verbose=yes	{prompt="Print process progress (yes or no)"}
bool    debug=no	{prompt="Print debugging info (yes or no)"}

struct *thartab

begin

    string apfib, smjd, sshift, rmode
    file tharfile, spthar, spec
    bool vb, ow, db, dt, first
    real mjd, mjd0, shift0, mjd1, mjd2, shift1, shift2

    rmode = refmode
    vb = verbose
    db = debug
    spec = spectrum

# Find aperture/binning/fiber size code to check ThAr spectra
    trsfib1 (spec, debug=db)
    mjd0 = trsfib1.mjdmid
    apfib = trsfib1.apfib
    if (db)
	print ("THARSHIFT1: ",spec," ",mjd," apfib=",apfib)

# Set up database file name for this configuration
    if (mode == "reference") {
	shift = 0.0
	return
	}

# Set up database file name for this configuration
    else {
	tharfile = "comp" // apfib // ".shift"
	if (!access (tharfile)) {
	    thartable (apfib=apfib,deltemp=yes,verbose=vb,debug=db)
	    if (thartable.outfile == "") {
		shift = 0.0
		refpath = ""
		return
		}
	    if (!access (tharfile)) {
		print ("THARSHIFT1: Cannot create ",tharfile)
		return
		}
	    }
	if (debug)
	    print ("THARSHIFT1: Reading ",tharfile)

	first = yes
	thartab = tharfile
	mjd1 = mjd0
	mjd2 = mjd0
	diff1 = 1.0
	diff2 = 1.0

# Read two header lines
	if (fscan (thartab, spthar) == EOF) {
	    if (db)
		print ("THARSHIFT1: cannot read line 1 of ",tharfile)
	    }
	if (fscan (thartab, spthar) == EOF) {
	    if (db)
		print ("THARSHIFT1: cannot read line 2 of ",tharfile)
	    }

# Read ThAr file name, MJD, and shift from each line
	while (fscan (thartab, spthar, smjd, refpath, sshift) != EOF) {
	    if (db)
		print ("THARSHIFT1: ",spthar," ", refpath, " ",mjd, " ", sshift)
	    mjd = real (smjd)
	    if (mjd > mjd0 && mjd0 - mjd < diff1) {
		diff1 = mjd0 - mjd
		shift1 = real (sshift)
		refpath = spthar
		}
	    else if (mjd > mjd0 && mjd - mjd0 < diff2) {
		diff2 = mjd - mjd0
		shift2 = real (sshift)
		refpath = spthar
		}
	    }

# Mode interp: Interpolate linearly between two closest ThAr spectra
    if (rmode == "interp") {
	if (mjd2 > mjd0 && mjd1 < mjd0) {
	    shift = shift1 + ((mjd0 - mjd1) / (mjd2 - mjd1)) * (shift2 - shift1)
	    if (debug)
		printf ("THARSHIFT1: shift= %.5f (%.5f,%.5f) MJD= %.5f (%.5f,%.5f)\n",
			shift,shift0,shift1,mjd,mjd0,mjd1)
	    return
	    }
	else
	    rmode = "closest"
	}

# Mode closest: Use shift from closest ThAr spectrum
    if (rmode == "closest") {
	if (mjd2 > mjd0 && mjd1 < mjd0) {
	    if (diff1 < diff2) {
		shift = shift1
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f at %.5f from %.f5\n",shift,mjd,mjd1)
		}
	    else if (diff2 < diff1) {
		shift = shift2
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f at %.5f from %.f5\n",shift,mjd,mjd2)
		}
	    else if (shift2 != shift1) {
		shift = shift1 + ((mjd0 - mjd1) / (mjd2 - mjd1)) * (shift2 - shift1)
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f (%.5f,%.5f) MJD= %.5f (%.5f,%.5f)\n",
			    shift,shift0,shift1,mjd0,mjd1,mjd2)
		}
	    else if (mjd2 > mjd0) {
		shift = shift2
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f at %.5f from %.f5\n",shift,mjd0,mjd2)
		}
	    else if (mjd1 > mjd0) {
		shift = shift1
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f at %.5f from %.f5\n",shift,mjd0,mjd1)
		}
	    else {
		shift = 0.0
		if (debug)
		    printf ("THARSHIFT1: shift= 0.0 at %.5f\n",shift,mjd0)
		}
	    }
	}
    else {
	print "THARSHIFT1: Mode ",rmode, " not implemented"
	shift = 0.0
	}
    if (vb) {
	printf ("%s pixel shift is %.5f at %.5f\n", spec, shift, mjd)
	}
    return
end

# May 23 2008	New task
# May 30 2008	Use trsfib to get center MJD of exposure
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
# Jul 18 2008	Add modes for shift interpolation
