# QTRES - Process a single TRES image from a raw data file to a spectrum
# July 2, 2009
# By Doug Mink, SAO Telescope Data Center

procedure qtres (image)

string	image=""	{prompt="Image or sequence number to process"}
string	datedir=""	{prompt="Date for image"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	datename=no	{prompt="Prepend observation date to file name (yes or no)"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	fixpreamp=no	{prompt="Fix preamp time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	flatten=yes	{prompt="Run TFLAT to Flatten image (yes or no)"}
string  normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string  normdir=""	{prompt="Directory for normalization file"}
string	aproot="flat"	{prompt="Root file name for aperture reference file"}
file	apdir="tresdata$" {prompt="Directory for aperture reference file"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
bool	disperse=yes	{prompt="Dispersion correct extracted spectra (yes or no)"}
string	comproot="comp"	{prompt="Root file name for calibration lamp file"}
file	compdir="tresdata$" {prompt="Directory for calibration lamp file"}
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
int	plotorder=1	{prompt="Echelle order to plot (No plot if zero"}
bool	deltemp=yes	{prompt="Delete intermediate image files (yes or no)"}
bool	delimage=no	{prompt="Delete processed image file (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	saveold=no	{prompt="Save intermediate image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

begin

file	ims, imin, impath, cdir
file	imout
string	imroot, ddir, croot
int	pl, n
bool	vb, dt, sv, sr, disp, delim, iscomp, dn

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	pl = plotorder
	dn = datename
	delim = delimage
	ddir = datedir
	imin = image
	cdir = compdir
	croot = comproot
	disp = disperse

	if (imin == "") {
	    print ("QTRES: Reduce a single TRES image in a different directory")
	    print ("Usage: qtres image with both amps to be corrected")
	    return
	    }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

# Copy image to current directory
	if (instdir != "") {
	    fixdir (instdir)
	    impath = fixdir.newdir

# If a number is given, prepend appropriate zeroes and wildcard
	    isnum (imin)
	    if (isnum.fpcode > 0) {
		n = int (imin)
		if (n < 10)
		    imin = "000" // n // ".*"
		else if (n < 100)
		    imin = "00" // n // ".*"
		else if (n < 1000)
		    imin = "0" // n // ".*"
		else
		    imin = "0" // n // ".*"
		}

# If file extension is missing, add wildcard
	    else {
		smatch (imin, ".fits")
		if (smatch.found == no)
		    imin = imin // "*"
		}
	    if (ddir != "")
		impath = impath // ddir // "/" // imin
	    else
		impath = impath // imin
	    filepath (impath)
	    if (!access (filepath.files)) {
		print ("QTRES: Cannot read file ",filepath.files)
		}
	    tres.filename (filepath.files)
	    if (dn && ddir != "")
		imout = ddir // "." // tres.filename.filename
	    else
		imout = tres.filename.filename
	    if (vb) {
		print ("QTRES: Copying file ",filepath.files," to ", imout)
		}
	    if (!access (imout))
		copy (impath, imout)
	    }
	else {
	    imout = imin
	    }

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.overscan = yes
	tproc.subbias = no
	tproc.subdark = no
	tproc.flatten = flatten
	tproc.extract = yes
	tproc.disperse = disperse
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.cosmic = no
	tproc.aproot = aproot
	tproc.apdir = apdir
	tproc.refroot = croot
	tproc.sumspec = "no"
	tproc.refdir = cdir
	tproc.recenter = recenter
	tproc.medshift = medshift
	tproc.deltemp = dt
	tproc.saveold = sv
	tproc.saveraw = sr
	tproc.verbose = vb
	tproc (imout)

	if (delim) {
	    delete (imout)
	    }

# Plot spectra if desired
	if (pl > 0) {
	    fileroot (imout)
	    imroot = fileroot.rootname
	    smatch (imroot, "COMP")
	    iscomp = smatch.found
	    ims = imroot // "_f1.ec.fits"
	    if (access (ims)) {
		if (iscomp)
		    tharplot (ims,pl)
		else if (pl > 0)
		    splot (ims, pl)
		else if (pl < 0)
		    trsplot (ims, -pl)
		}
	    ims = imroot // "_f2.ec.fits"
	    if (access (ims)) {
		if (iscomp)
		    tharplot (ims,pl)
		else if (pl > 0)
		    splot (ims, pl)
		else if (pl < 0)
		    trsplot (ims, -pl)
		}
	    ims = imroot // "_f3.ec.fits"
	    if (access (ims)) {
		if (iscomp)
		    tharplot (ims,pl)
		else if (pl > 0)
		    splot (ims, pl)
		else if (pl < 0)
		    trsplot (ims, -pl)
		}
	    }
	if (tcheck.rawdir)
	    tdata (datedir=tcheck.datedir)
end

# Oct 12 2007	New task for quick look
# Dec 11 2007	Remove old declaration of instdir from before it was a parameter

# Jan 16 2008	Add fixpreamp parameter
# Jan 17 2008	Use tproc instead of tpreproc and tflatten
# Jan 23 2008	Add plotorder parameter to plot spectrum at end
# Jan 23 2008	Add recenter to make recentering during extraction optional
# Jan 23 2008	Add comproot parameter and wavelength calibration
# Jan 28 2008	Add disperse parameter to make wavelength optional
# Jan 30 2008	Use tdisp to add wavelength solution
# Jan 30 2008	Add parameters apdir and aproot for extraction templates
# Feb 12 2008	Use TPROC for everything; drop calls to textract and tdisperse
# Feb 13 2008	Change parameter imdate to datedir
# Mar 26 2008	Change saveraw and saveold default to no, plotorder to 23
# Apr 29 2008	Add delimage to allow deletion of processed image file
# Aug 13 2008	Name files _f1 _f2 _f3 instead of _fib1 _fib2 _fib3
# Sep  9 2008	Plot ThAr spectra using THARPLOT instead of SPLOT
# Sep 19 2008	Add medshift parameter which gets passed through TPROC to TCAL1
# Sep 19 2008	Add datename to make prepended date in file name optional
# Oct 14 2008	Check to make sure we're not processing in raw data directory
# Oct 14 2008	Make default to save raw files
# Oct 16 2008	From raw directory, switch to quick-look directory and back
# Oct 17 2008	Change default to not delete processed pre-extraction image
# Nov 13 2008	Specify task tres.filename

# Apr 10 2009	Make default normdir = "" (working directory)
# Apr 10 2009	Call TRSPLOT instead of SPLOT for nonCOMP spectra
# Apr 16 2009	If plotorder > 0, plot object spectra with splot
# Apr 16 2009	If plotorder < 0, plot object spectra with trsplot -plotorder
# Jul  2 2009	Set overscan and subbias to subtract overscan
