# TBIAS1 -- Subtract bias image from TRES image
# By Doug Mink, SAO Telescope Data Center
# June 19, 2009

procedure tbias1 (image, biasfile)

string	image = ""	{prompt="Input multispec image to correct"}
string	biasfile=""	{prompt="Dark image to subtract after scaling"}

string	biasdir=""	{prompt="Directory for bias image file"}
bool	saveold = yes	{prompt="Save original file to Unflat/ (y or n)"}
bool	deltemp=yes	{prompt="Delete temporary files (yes or no"}
bool	verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb, sv, dt, makename
	file imin, imout, imroot, imnorm, newfile, templist, bdir, bfile
	int lroot
	string history, ims, tchar, flatref, imref, bpath
	real expd, expi, ratio

	imin = image
	vb = verbose
	sv = saveold
	dt = deltemp
	bfile = biasfile
	if (bfile == "")
	    makename=yes
	else
	    makename = no
	bdir = biasdir
	fixdir (bdir)
	bdir = fixdir.newdir

# Do not subtract bias image if it has already been subtracted
	imgetp.default = "0"
	imgetp (imin, "TBIAS")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("TBIAS1: ", imin, " has been processed")
		print ("TBIAS = ", imgetp.value)
		}
	    return
	    }

# Remove extension from input file, if one is present, and make output file name
	smatch (imin, ".fits")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".sbias.fits"
	    }
	else {
	    imout = imin // ".sbias.fits"
	    imin = imin // ".fits"
	    }

# Add binning code and FITS extension to bias file if it is missing
	if (makename) {
	    tfib1 (imin, specdir="")
	    bfile = "bias" // tfib1.binning // ".fits"
	    }
	else {
	    smatch (bfile, ".fits")
	    if (!smatch.found) {
		bfile = bfile // ".fits"
		}
	    }

# Check for the existence of the normalization spectrum locally
	if (!access (bfile)) {
#	    print ("TBIAS1: Bias file ",bfile," not found")
	    if (bdir == "")
		bdir = "tresdata$"
	    bfile = bdir // bfile
	    }
	if (!access (bfile)) {
	    print ("TBIAS1: Bias file ",bfile," not found")
	    return
	    }

# Subtract bias image
	imarith (imin, "-", bfile, imout)

# Move original file to Unbias directory
	if (sv) {
	    if (!access ("Unbias/")) {
		mkdir ("Unbias")
		}
	    newfile = "Unbias/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    movefiles (imin, "Unbias")
	    if (vb) {
		print ("TBIAS1: Original ", imin, " => Unbias/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	rename (imout, imin)

# Add history line including time
	gettime (0.0, timediff=99)
	history = gettime.date // " " // gettime.time // " Bias file subtracted"
	hedit (imin, "TBIAS", history, add+, verify-)
	imgetp.default = ""
	imgetp (bfile, "BIASFILE")
	bpath = imgetp.value
        hedit (imin, "TBFILE", bpath, add+, verify-)

end

# Jun 19 2009	New task based on TDARK1
