# File tres/Scripts/tblaze1.cl
# April 28, 2009
# By Doug Mink

#  TBLAZE1 -- Remove echelle blaze function from a spectrum

procedure tblaze1 (spectrum)

string	spectrum=""	{prompt="Image to correct"}

string	blzroot="flat"	{prompt="Root name for normalized blaze function spectrum"}
string	blzdir="tresdata$" {prompt="Directory for blaze function spectrum"}
bool	saveold = yes	{prompt="Save original file to Noblaze/ (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print extra information (y or n)"}

begin

    string imin, history
    file blzpath	# Path name for normalized blaze function spectrum
    file impix, bdir, newfile, blzfile
    bool vb, db

    imin = spectrum
    vb = verbose
    db = debug
    fixdir (blzdir)
    bdir = fixdir.newdir

# Skip this file if it has already had its blaze function removed
    imgetp.default = "0"
    imgetp (imin, "TBLAZE")
    if (imgetp.value != "0") {
	if (vb) {
	    print ("TBLAZE1: ", imin, " has had its blaze function removed")
	    print ("TBLAZE = ", imgetp.value)
	    }
	return
	}

    tfib1 (imin)
    blzfile = blzroot // tfib1.apfib // "n.ec.fits"

# If blaze file for this file does not exist, exit
    if (!access (blzfile)) {
	blzpath = bdir // blzfile
	if (!access (blzpath)) {
	    print ("TBLAZE1: Cannot read correction file ", blzpath)
	    return
	    }
	}
    else
	blzpath = blzfile

# Make up output file name
    fileroot (imin)
    fileroot (fileroot.rootname)
    impix = fileroot.rootname // ".blz.ec.fits"

# Divide spectrum by blaze function in pixel space
    imarith (imin, "/", blzpath, impix)

# Add history line including time
    gettime (0.0, timediff=99)
    history = gettime.date // " " // gettime.time
    history = history // " blaze function removed using " // blzpath
    hedit (impix, "TBLAZE", history, add+, verify-)

# Move original file to Noblaze directory
    if (saveold) {
	if (!access ("Noblaze/")) {
	    mkdir ("Noblaze")
	    if (vb)
		print ("TBLAZE1: Making directory Noblaze/")
	    }
	newfile = "Noblaze/" // imin
	if (access (newfile)) {
	    imdelete (newfile)
	    }
	rename (imin, newfile)
	if (vb) {
	    print ("TBLAZE1: Original ", imin, " => Noblaze/")
	    }
	}
    else {
	delete (imin)
	}
    rename (impix, imin)

end

# Apr 24 2009	New task structurally based on TDISP1
# Apr 28 2009	Drop blzpath from parameter list; it is really a local variable
