# File tres/Scripts/tscat1.cl
# By Doug Mink
# June 10, 2009

# TSCAT1 -- Use a mask file to remove the spectrum from a TRES image,
#           fit the background with a surface, and subtract it

procedure tscat1 (imfile)

file	imfile=""	{prompt="Input TRES image to correct"}
string  maskroot="flat"	{prompt="Root file name for mask file"}
string  maskdir="tresdata$"	{prompt="Directory for mask file if not local"}
string	function="legendre"	{prompt="Function to fit to surface (legendre|chebyshev|spline3)"}
int	xorder = 2	{prompt="Order of function in x"}
int	yorder = 2	{prompt="Order of function in y"}
bool	crossterms = yes	{prompt="Include cross-terms for polynomials?"}
bool    saveold = yes   {prompt="Save original file to Unscat/ (yes or no)"}
bool	deltemp = yes	{prompt="Delete intermediate files (yes or no)"}
bool	verbose=yes	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print output (yes or no)"}

begin

    bool vb, db, sv, dt
    int i, io, ncoeff, icoeff, nfib
    file mdir, maskpath, newfile, fillfile
    file infile, maskfile, fitfile, backfile, ifile, outfile
    string outroot, outroot1, outroot2, filepath, dataroot, apfib
    string history
    int oldap, newap

    vb = verbose
    db = debug
    sv = saveold
    dt = deltemp
    mdir = maskdir

    tcheck

# Quit if input file is not there
    infile = imfile
    if (!access (infile)) {
	print ("TSCAT1: Input file ",infile, " does not exist")
	return
	}

# Find fiber information for this file
    tfib1 (infile)
    if (tfib1.fiber == "1" || tfib1.fiber == "2")
	apfib = tfib1.apsize // tfib1.binning // "12"
    else
	apfib = tfib1.apfib

# Make mask file name from root and apfib
    maskfile = maskroot // apfib // ".mask.fits"

# Quit if mask file does not exist
    if (!access (maskfile)) {
	if (mdir != "") {
	    fixdir (mdir)
	    mdir = fixdir.dirpath
	    maskpath = mdir // maskfile
	    }
	else
	    maskpath = "tresdata$" // maskfile
	}
    else
	maskpath = maskfile
    if (!access (maskpath)) {
	printf ("TSCAT1: Mask file %s does not exist\n", maskpath)
	return
	}
    if (vb)
	print ("TSCAT1: Using mask file ", maskpath)

# Make name for spectrum-free file and delete it if it already exists
    fileroot (infile)
    backfile = fileroot.rootname // ".back.fits"
    if (access (backfile))
	delete (backfile)

# Remove spectrum from input file
    imarith.title = "TRES Background"
    imarith.divzero = 0.
    imarith.pixtype = "real"
    imarith.calctype = "real"
    imarith.noact = no
    imarith (infile, "*", maskpath, backfile, verbose=vb)
    hedit (backfile, "BLANK", 0.0, add+, verify=db)
    if (vb) {
	print ("TSCAT1: Spectrum removed from ",infile," using ",maskpath)
	}

# Make name for background fill file and delete it if it already exists
    fillfile = fileroot.rootname // ".bfill.fits"
#    if (access (fillfile))
#	delete (fillfile)

# Interpolate background across spectrum
#    timfill.error = 0
#    timfill (backfile, fillfile, verbose=vb, debug=db)

# Make name for background fit file and delete it if it already exists
    fitfile = fileroot.rootname // ".bfit.fits"
    if (access (fitfile))
	delete (fitfile)

# Fit background with a surface
    imsurfit.type_output = "fit"
    imsurfit.function = function
    imsurfit.cross_terms = crossterms
    imsurfit.xmedian = 1
    imsurfit.ymedian = 1
    imsurfit.median_percent = 50.
    imsurfit.lower = 0.
    imsurfit.upper = 0.
    imsurfit.ngrow = 0
    imsurfit.niter = 0
    imsurfit.regions = "all"
    imsurfit.rows = "*"
    imsurfit.columns = "*"
    imsurfit.border = "50"
    imsurfit.sections = ""
    imsurfit.circle = ""
    imsurfit.div_min = INDEF
#    imsurfit (fillfile, fitfile, xorder, yorder)
    imsurfit (backfile, fitfile, xorder, yorder)
    if (vb) {
	print ("TSCAT1: ",xorder,"x",yorder," ",function," fit to ",backfile)
	}

# Make name for image - fit file and delete it if it already exists
    outfile = fileroot.rootname // ".res.fits"
    if (access (outfile))
	delete (outfile)

# Subtract background fit from the original image
    imarith (infile, "-", fitfile, outfile)
    if (vb) {
	print ("TSCAT1: ",fitfile," subtracted from ",infile)
	}

# Add history line including time
    gettime (0.0, timediff=99)
    history = gettime.date // " " // gettime.time
    history = history // " scattered light removed"
    hedit (outfile, "TSCAT", history, add+, verify=db)

# Move original file to Unscat directory
    if (sv) {
	if (!access ("Unscat/")) {
	    mkdir ("Unscat")
	    if (vb)
		print ("TSCAT1: Making directory Unscat/")
	    }
	newfile = "Unscat/" // infile
	if (access (newfile)) {
	    imdelete (newfile)
	    }
	rename (infile, newfile)
	if (vb) {
	    print ("TSCAT1: Original ", infile, " => Unscat/")
	    }
	}
    else {
	delete (infile)
	}
    rename (outfile, infile)

    if (dt) {
	delete (backfile)
	delete (fitfile)
	if (access (fillfile))
	    delete (fillfile)
	}

end

# Apr 21 2009	New task to fit background surface and subtract it
# Apr 28 2009	If only one fiber in input, use mask file with two fibers
# May 18 2009	Do not fix directory name if it is null
# Jun 10 2009	Use tresdata as default directory if mask file not found
