# File tres/Scripts/tsetbcv.cl
# By Doug Mink, SAO Telescope Data Center
# May 20, 2009

# TSETBCV -- Add barycentric velocity correction (BCV) to image header

procedure tsetbcv (images)

string	images=""	{prompt="Files for which to set barycentric velocity correction"}
bool	writebcv=yes	{prompt="Write BCV and JD to file (yes or no)"}
bool	verbose=no	{prompt="Display heliocentric and barycentric correction (yes or no)"}
bool	deltemp=yes	{prompt="Delete temporary list file (yes or no)"}
bool	debug=no	{prompt="Show intermediate computed values (yes or no)"}

struct *bcvfiles

begin
    file imin		# File for which to set barycentric velocity correction
    string ims
    file templist
    string imtype
    bool vb, db, dt, wb

    ims = images
    vb = verbose
    db = debug
    dt = deltemp
    wb = writebcv

    if (!defpac ("rvsao"))
	rvsao (motd=no)

#  Make temporary list of input files
    templist = mktemp ("tempbcvp") // ".list"
    files (ims, >templist)

    bcvfiles = templist
    while (fscan (bcvfiles,imin) != EOF) {

	imgetp.default = ""
	imgetp (imin, "IMAGETYP")
	imtype = imgetp.value
	if (imtype == "OBJECT") {
	    bcvcorr.spectra = ""
	    bcvcorr.specnum = "0"
	    bcvcorr.specdir = ""
	    bcvcorr.specsky = no
	    bcvcorr.keyra = "RA"
	    bcvcorr.keydec = "DEC"
	    bcvcorr.keyeqnx = "EPOCH"
	    bcvcorr.ra = "INDEF"
	    bcvcorr.dec = "INDEF"
	    bcvcorr.equinox = "INDEF"
	    bcvcorr.keydate = "DATE-OBS"
	    bcvcorr.keytime = "UTOPEN"
	    bcvcorr.keywhen = "start"
	    bcvcorr.keyexp = "EXPTIME"
	    bcvcorr.keyhjd = ""
	    bcvcorr.hjd = 0.
	    bcvcorr.gjd = 0.
	    bcvcorr.year = INDEF
	    bcvcorr.month = INDEF
	    bcvcorr.day = INDEF
	    bcvcorr.ut = INDEF
	    bcvcorr.obsname = ""
	    bcvcorr.keyobs = "SITENAME"
	    bcvcorr.keylat = "SITELAT"
	    bcvcorr.keylong = "SITELONG"
	    bcvcorr.keyalt = "SITEELEV"
	    bcvcorr.obslong = INDEF
	    bcvcorr.obslat = INDEF
	    bcvcorr.obsalt = INDEF
	    bcvcorr.savebcv = wb
	    bcvcorr.savejd = wb
	    bcvcorr.bcv = INDEF
	    bcvcorr.hcv = INDEF
	    bcvcorr.verbose = vb
	    bcvcorr.printmode = 1
	    bcvcorr.debug = db

# Check file name for twilight sky
	    smatch (imin, "SKY")
	    if (!smatch.found) {
		smatch (imin, "sky")
		if (!smatch.found) {
		    smatch (imin, "Sky")
		    if (!smatch.found) {
			smatch (imin, "SOLAR")
			}
		    }
		}

# If sky, use earth velocity relative to the sun
	    if (smatch.found) {
		if (vb)
		    print ("TSETBCV: solar BCV because object is ", imgetp.value)
		bcvcorr (imin,specsky=yes)
		}

# Otherwise, use vector of earth velocity relative to the sun toward object
	    else {
		bcvcorr (imin,specsky=no)
		}
	    }
	}

    if (dt)
	delete (templist)

    bcvfiles = ""
    return
end
# Oct 27 2008	New program
# Oct 29 2008	Load rvsao if it is not already loaded
# Dec  4 2008	Add deltemp parameter

# Apr 16 2009	Correct to sky velocity if OBJECT is SOLAR
# May 20 2009	Add writebcv parameter to write BCV and JD to file
