# TXSTAT - Run imstat on several files, one image extension at a time
# July 2, 2009
# By Doug Mink, SAO Telescope Data Center

procedure txstat (images)

string	images=""	{prompt="Images to check"}

string	fields="image,npix,mean,stddev,min,max" {prompt="Fields to be printed"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print error messages (y or n)"}
bool	debug=no	{prompt="Print process progress (y or n)"}

begin

int	iext
file	templist, imin, tempfile
string	ims, imsec, imlist, flds, nextim
bool	vb, first, db

	vb = verbose
	ims = images
	db = debug
	flds = fields
	first = yes

	if (ims == "") {
	    print ("TXSTAT:  Run imstat on a list of images one amp at a time")
	    print ("Usage: txstat <input images with 2 extensions>")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (substr (ims,1,1) != '@')
		ims = "@" // ims
	    }

#  Make temporary list of input files
	trsfiles.copyfile = no
	trsfiles.adddate = no
	trsfiles.keeppath = yes
	trsfiles.extension = ".fits"
	trsfiles (ims)
	trsfiles (ims)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("TXSTAT: No files found: ", ims)
	    return
	    }
	templist = trsfiles.tempout
	tempfile = mktemp ("tempfile") // ".list"
	list = templist
        while (fscan (list,imin) != EOF) {
	    for (iext = 1; iext < 3; iext = iext + 1) {

# Run imstat on each extension of each image
		nextim = imin // "[" // iext // "]"
		if (db)
		    print ("TXSTAT: Statistics for ",imin," -> ",nextim)
		if (first) {
		    print (nextim, >tempfile)
		    first = no
		    }
		else
		    print (nextim, >>tempfile)
		
		}
	    }
	tistat ("@"//tempfile, fields=flds, print=yes)

	if (deltemp) {
	    delete (templist)
	    delete (tempfile)
	    delete (trsfiles.tempin)
	    }
end

# Jan 15 2008	New program to process list of multiextension TRES images
# Feb 11 2008	Change name of temporary file from txstatlist* to tempstatlist*
# Oct  6 2008	Change order of loops to all extensions for each image
# Oct  6 2008	Write files to temporary file instead of string
# Nov 17 2008	Write temporary files to /tmp

# Mar  4 2009	Read from raw data directory using trsfiles
# Mar  9 2009	Return if no files found from list
# May 12 2009	Use instdir and datedir from TRSFILES
# May 12 2009	Add debug parameter
# Jul  2 2009	Set all parameters of TRSFILES
