include	<mach.h>
include	<imhdr.h>
include	<gset.h>

# THISTOGRAM -- Compute and plot the histogram of an image.

procedure t_thistogram()

real	z1, z2, z1temp, z2temp, rindef
bool	listout,lognum, intbin
long	v[IM_MAXDIM]
int	npix, nbins, z1i, z2i, i
pointer gp, im, sp, hgm, hgmr, buf, image, device
char	title[SZ_LINE]

bool	clgetb(), fp_equalr()
int	clgeti()
real	clgetr()
int	imgnlr(), imgnli()
pointer	immap(), gopen()

begin
	call smark (sp)
	call salloc (image,   SZ_LINE,  TY_CHAR)
	call salloc (device,  SZ_FNAME, TY_CHAR)
	rindef = INDEFR

# Get the image name.
	call clgstr ("image", Memc[image], SZ_LINE)
	im = immap (Memc[image], READ_ONLY, 0)
	npix = IM_LEN(im,1)

# Output can be either a list or a plot.
	listout = clgetb ("listout")
	if (!listout)
	    call clgstr ("device", Memc[device], SZ_FNAME)

# Output can have optional log-scaled y axis
	lognum = clgetb ("logy")

	z1 = clgetr ("z1")
	z2 = clgetr ("z2")
	z1temp = rindef
	z2temp = rindef

# Get range of values in the image if parameters are INDEF
	if (z1 == rindef) {
	    call im_minmax (im, z1temp, z2temp)
	    z1 = z1temp
	    }
	if (z2 == INDEF) {
	    if (z2temp == rindef)
	        call im_minmax (im, z1temp, z2temp)
	    z2 = z2temp
	    }

# Use integers for graph and binning limits
	z1i = nint (z1)
	z2i = nint (z2)

# Flag to force bins to be of integral width
	intbin = clgetb ("intbin")

# Get number of bins in the histogram
	if (intbin)
	    nbins = z2i - z1i + 1
	else
	    nbins = clgeti ("nbins")

# Allocate buffers for histogram
	call salloc (hgm,  nbins, TY_INT)
	call salloc (hgmr, nbins, TY_REAL)

# Initialize histogram and image line vector
	call aclri (Memi[hgm], nbins)
	call amovkl (long(1), v, IM_MAXDIM)

# Read successive lines of the image into memory and accumulate the histogram
	switch (IM_PIXTYPE(im)) {
	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
	    # Test for constant valued image, which causes zero divide in ahgm.
	    if (z1i == z2i) {
	        call eprintf (
		    "Warning: Constant valued image `%s' has no data range.\n")
		    call pargstr (Memc[image])
	        call imunmap (im)
	        call sfree (sp)
	        return
	    }

	    while (imgnli (im, buf, v) != EOF) 
		call ahgmi (Memi[buf], npix, Memi[hgm], nbins, z1i, z2i)

	default:
	    # Test for constant valued image, which causes zero divide in ahgm.
	    if (fp_equalr (z1, z2)) {
	        call eprintf (
		    "Warning: Constant valued image `%s' has no data range.\n")
		    call pargstr (Memc[image])
	        call imunmap (im)
	        call sfree (sp)
	        return
	    }

	    while (imgnlr (im, buf, v) != EOF)
		call ahgmr (Memr[buf], npix, Memi[hgm], nbins,  z1,  z2)
	}

# List the histogram
	if (listout) {
	    do i = 1, nbins {
		call printf ("%3d %d\n")
		    call pargi (i)
		    call pargi (Memi[hgm+i-1])
	    }

# Plot the histogram
	} else {
	    gp = gopen (Memc[device], NEW_FILE, STDGRAPH)
	    if (lognum) {
		call gseti (gp, G_YTRAN, GW_LOG) }
	    else {
		call gseti (gp, G_YTRAN, GW_LINEAR) }
	    call achtir (Memi[hgm], Memr[hgmr], nbins)
	    call sprintf (title,SZ_LINE,"%s: %s")
		call pargstr (Memc[image])
		call pargstr (IM_TITLE(im))
	    call gploth (gp, Memr[hgmr], nbins, z1, z2, title)
	    call gclose (gp)
	}

# Shutdown
	call imunmap (im)
	call sfree (sp)
end

# Jun 13 1988	Modified to make log scaling optional

# Sep 14 1989	Modified to print file name optional

# Jun 23 2009	Copied from saotdc.itools.ihistogram
